@@ -2,8 +2,8 @@
 # Licensed under the terms of the Apache License, Version 2.0. See the LICENSE file associated with the project for terms.
 """Lightweight :term:`computation` graphs for Python."""
 
-__version__ = "10.2.0"
-__release_date__ = "16 Aug 2020, 12:15"
+__version__ = "10.3.0"
+__release_date__ = "22 Aug 2020, 18:24"
 __title__ = "graphtik"
 __summary__ = __doc__.splitlines()[0]
 __license__ = "Apache-2.0"
@@ -13,8 +13,9 @@ __author__ = "hnguyen, ankostis"  # chronologically ordered
 
 from .base import AbortedException, IncompleteExecutionError
 from .modifier import (
-    accessor,
     modify,
+    hcat,
+    implicit,
     keyword,
     optional,
     sfx,
@@ -23,6 +24,7 @@ from .modifier import (
     sfxed_varargs,
     vararg,
     varargs,
+    vcat,
 )
 from .fnop import NO_RESULT, NO_RESULT_BUT_SFX, operation
 from .pipeline import compose
