@@ -67,6 +67,35 @@ def _isDebugLogging():
     return log.isEnabledFor(logging.DEBUG)
 
 
+def _update_outputs_grouped_by_accessor(op_layer, outputs: Mapping):
+    """
+    Mass update values on the :term:`solution layer` for the given `op`
+
+    ... respecting any :attr:`._accessor.update` term:`accessor`\\s.
+
+    :param op_layer:
+        where to update values
+    :param outputs:
+        what to update
+    """
+    accessors = [get_accessor(o) for o in outputs]
+    if any(accessors):
+        ## Group out-values by their `update` accessor (or None).
+        update_groups = defaultdict(list)
+        for ac, kv in zip(accessors, outputs.items()):
+            update_groups[ac and ac.update].append(kv)
+
+        ## Values without :attr:`._accessor.update` are to be set one-by-one,
+        #  further below.
+        outputs = update_groups.pop(None, None)
+
+        for upd, pairs in update_groups.items():
+            upd(op_layer, pairs)
+
+    if outputs:
+        op_layer.update(outputs)
+
+
 class Solution(ChainMap, Plottable):
     """
     The :term:`solution` chain-map and execution state (e.g. :term:`overwrite` or :term:`canceled operation`)
@@ -75,49 +104,67 @@ class Solution(ChainMap, Plottable):
     for each operation executed, +1 for the user inputs.
     """
 
+    #: Command :term:`solution layer`, by
+    #:  default, false if not any jsonp in dependencies.
+    #:
+    #: See :attr:`executed` below
+    is_layered: bool
+    #: A dictionary with keys the operations executed,
+    #: and values their :term:`layer`, or status:
+    #:
+    #: - no key:                not executed yet
+    #: - value == dict:         execution ok, produced those outputs
+    #: - value == Exception:    execution failed
+    #:
+    #: Keys are ordered as operations executed (last, most recently executed).
+    #:
+    #: When :attr:`is_layered`, its value-dicts are inserted, in reverse order,
+    #: into my :attr:`maps` (from chain-map).
+    executed: OpMap = {}
+    #: A map of {rescheduled operation -> dynamically pruned ops, downstream}.
+    broken: Mapping[Operation, Operation] = {}
+    #: A {op, prune-explanation} dictionary with :term:`canceled operation`\\s
+    #: due to upstream failures.
+    canceled: OpMap = {}
+    elapsed_ms = {}
+    #: A unique identifier to distinguish separate flows in execution logs.
+    solid: str
+    # FIXME: SPURIOUS dag reversals on multi-threaded runs (see below next assertion)!
+    #: Cloned from `plan` will be modified, by removing the downstream edges of:
+    #:
+    #: - any partial outputs not provided, or
+    #: - all `provides` of failed operations.
+    dag: nx.DiGraph
+    #: the plan that produced this solution
+    plan = "ExecutionPlan"
+    # optimization for the expensive :attr:`.overwrites` dictionary
+    _overwrites_cache = None
+
     def __init__(
         self,
         plan,
         input_values: dict,
-        pre_callback: Callable[["OpCb"], None] = None,
-        layered_solution=None,
+        callbacks: Tuple[Callable[["OpCb"], None], Callable[["OpCb"], None]] = None,
+        is_layered=None,
     ):
-        self.pre_callback = pre_callback
-        layered_solution = first_solid(is_layered_solution(), layered_solution)
-        if layered_solution is None:
-            layered_solution = not any(get_jsonp(d) for d in plan.net.data)
+        super().__init__(input_values)
+        self.callbacks = callbacks
+        is_layered = first_solid(is_layered_solution(), is_layered)
 
-        #: :term:`layer` map of operations to their results
-        #: (initially empty dicts).
-        #: The result dictionaries pre-populate this (self) chainmap,
-        #: with the 1st map (wins all reads) the last operation executed,
-        #: the last one the `input_values` dict.
-        #:
-        #: If network contains :term:`jsonp` dependencies, by default
-        #: layers are disabled, and this is None.
-        self._layers: Optional[Mapping] = (
-            {op: {} for op in yield_ops(reversed(plan.steps))}
-            if layered_solution
-            else None
-        )
-        maps = self._layers.values() if self._layers else ()
-        super().__init__(*maps, input_values)
+        ##: By default, disable layers if network contains :term:`jsonp` dependencies.
+        #
+        if is_layered is None:
+            is_layered = not any(get_jsonp(d) for d in plan.net.data)
+        self.is_layered = is_layered
+
+        #: Preserve name-list to account overwrites when non-:term:`layer`\ed`.
+        self._initial_inputs = input_values if self.is_layered else dict(input_values)
 
-        #: the plan that produced this solution
         self.plan = plan
-        #: A dictionary with keys the operations executed, and values their status:
-        #:
-        #: - no key:            not executed yet
-        #: - value None:        execution ok
-        #: - value Exception:   execution failed
-        #: - value Collection:  canceled provides
-        self.executed: OpMap = {}
-        # not iterated, order unimportant, but ...
-        #: A {op, prune-explanation} dictionary with :term:`canceled operation`\\s
-        #: due to upstream failures.
-        self.canceled: OpMap = {}
+        self.executed: Mapping[Operation, dict] = {}
+        self.canceled = {}
+        self.broken = {}
         self.elapsed_ms = {}
-        #: A unique identifier to distinguish separate flows in execution logs.
         self.solid = "%X" % random.randint(0, 2 ** 16)
 
         ## Cache context-var flags.
@@ -127,34 +174,38 @@ class Solution(ChainMap, Plottable):
         self.is_parallel = is_parallel_tasks()
         self.is_marshal = is_marshal_tasks()
 
-        #: Cloned from `plan` will be modified, by removing the downstream edges of:
-        #:
-        #: - any partial outputs not provided, or
-        #: - all `provides` of failed operations.
-        #:
-        #: FIXME: SPURIOUS dag reversals on multi-threaded runs (see below next assertion)!
+        # FIXME: SPURIOUS dag reversals on multi-threaded runs (see below next assertion)!
         self.dag = plan.dag.copy()
         # assert next(iter(dag.edges))[0] == next(iter(plan.dag.edges))[0]:
 
     def copy(self):
-        """Deep-copy user's `input_data` and pass the rest intoa new Solution. """
+        """Deep-copy user's `input_data` and pass the rest into a new Solution. """
         named_inputs = dict(self.maps[-1])
         clone = type(self)(
             self.plan,
             named_inputs,
-            self.pre_callback,
+            self.callbacks,
             # Specially handled below bc user's `layered_solution` arg is lost.
-            layered_solution=None,
+            is_layered=self.is_layered,
         )
-        if self._layers is None:
-            clone._layers = None
 
+        ## replicate "layers"" machinery.
+        #
         props = (
-            "maps executed canceled elapsed_ms solid"
-            " is_endurance is_reschedule is_parallel is_marshal dag"
+            "is_layered is_endurance is_reschedule is_parallel is_marshal dag"
+            " _initial_inputs executed canceled broken elapsed_ms"
         ).split()
+
         for p in props:
-            setattr(clone, p, getattr(self, p))
+            val = getattr(self, p)
+            if isinstance(val, dict):
+                val = dict(val)
+            setattr(clone, p, val)
+
+        ## Replicate layer setup in constructor, here
+        #
+        executed_ok = reversed(self.layers) if self.is_layered else ()
+        clone.maps = [*executed_ok, named_inputs]
 
         return clone
 
@@ -168,7 +219,15 @@ class Solution(ChainMap, Plottable):
             return f"{{{items}}}"
 
     def debugstr(self):
-        return f"{type(self).__name__}({dict(self)}, {self.plan}, {self.executed}, {self._layers})"
+        return (
+            f"{type(self).__name__}({dict(self)}, {self.plan}, {self.executed}, "
+            f"is_layered={self.is_layered})"
+        )
+
+    @property
+    def layers(self) -> List[OpMap]:
+        """Outputs by operation, in execution order (last, most recently executed). """
+        return [v for v in self.executed.values() if not isinstance(v, Exception)]
 
     def _reschedule(self, dag, reason, op):
         """
@@ -204,31 +263,6 @@ class Solution(ChainMap, Plottable):
                 op.name,
             )
 
-    def _update_op_outs(self, op, outputs: Mapping):
-        """
-        Mass update values on the :term:`solution layer` for the given `op`.
-
-        Observes any :term:`accessor`\\s in the k
-        A separate method to allow subclasses with custom accessor logic.
-        """
-        op_layer = self._layers[op] if self._layers else self.maps[-1]
-        accessors = [get_accessor(o) for o in outputs]
-        if any(accessors):
-            ## Group out-values by their `update` accessor (or None).
-            update_groups = defaultdict(list)
-            for ac, kv in zip(accessors, outputs.items()):
-                update_groups[ac and ac.update].append(kv)
-
-            ## Values without :attr:`._accessor.update` are to be set one-by-one,
-            #  further below.
-            outputs = update_groups.pop(None, None)
-
-            for upd, pairs in update_groups.items():
-                upd(op_layer, pairs)
-
-        if outputs:
-            op_layer.update(outputs)
-
     def __contains__(self, key):
         acc = acc_contains(key)
         return any(acc(m, key) for m in self.maps)
@@ -242,7 +276,13 @@ class Solution(ChainMap, Plottable):
                 pass
         return self.__missing__(key)
 
+    def __setitem__(self, key, val):
+        self._overwrites_cache = None
+        super().__setitem__(key, val)
+
     def __delitem__(self, key):
+        self._overwrites_cache = None
+
         acc = acc_contains(key)
         matches = [m for m in self.maps if acc(m, key)]
         if not matches:
@@ -252,6 +292,32 @@ class Solution(ChainMap, Plottable):
         for m in matches:
             acc(m, key)
 
+        ## Delete it from extra places when non-layered.
+        #
+        if not self.is_layered:
+            for m in self.layers:
+                m.pop(key, None)
+
+            self._initial_inputs.pop(key, None)
+
+    def _populate_op_layer_with_outputs(self, op, outputs) -> dict:
+        """
+        Populate a new layer for `op` with `outputs` , or use `named_inputs` (if non-layered).
+        """
+        self._overwrites_cache = None
+
+        if self.is_layered:
+            op_layer = {}
+            self.maps.insert(0, op_layer)
+            self.executed[op] = op_layer
+        else:
+            op_layer = self.maps[-1]
+            # Update just they keys, the values are in `input_names`.
+            self.executed[op] = outputs
+
+        if outputs:
+            _update_outputs_grouped_by_accessor(op_layer, outputs)
+
     def operation_executed(self, op, outputs):
         """
         Invoked once per operation, with its results.
@@ -274,9 +340,7 @@ class Solution(ChainMap, Plottable):
                 return ()
             return dep_singularized(dep)
 
-        self._update_op_outs(op, outputs)
-        self.executed[op] = None
-
+        self._populate_op_layer_with_outputs(op, outputs)
         if first_solid(self.is_reschedule, getattr(op, "rescheduled", None)):
             ## Find which provides have been broken?
             #
@@ -301,7 +365,7 @@ class Solution(ChainMap, Plottable):
                 dag.remove_edges_from((op, out) for out in outs_to_break)
                 self._reschedule(dag, "rescheduled", op)
                 # list used by `check_if_incomplete()`
-                self.executed[op] = outs_to_break
+                self.broken[op] = outs_to_break
 
     def operation_failed(self, op, ex):
         """
@@ -316,7 +380,9 @@ class Solution(ChainMap, Plottable):
         self._reschedule(dag, "failure of", op)
 
     def is_failed(self, op):
-        return isinstance(self.executed.get(op), Exception)
+        """returns Non(not executed), False(ok), Exception(failed)"""
+        exe = self.executed.get(op)
+        return exe and isinstance(exe, Exception) and exe
 
     @property
     def overwrites(self) -> Mapping[Any, List]:
@@ -325,15 +391,27 @@ class Solution(ChainMap, Plottable):
 
         A "virtual" property to a dictionary with keys the names of values that
         exist more than once, and values, all those values in a list, ordered
-        in reverse compute order (1st is the last one computed).
+        in reverse compute order (1st is the last one computed, last (any) given-inputs).
         """
-        maps = self.maps
+        if self._overwrites_cache is not None:
+            return self._overwrites_cache
+
+        if self.is_layered:
+            maps = self.maps
+        else:
+            maps = [*reversed(self.layers), self._initial_inputs]
         dd = defaultdict(list)
         for d in maps:
             for k, v in d.items():
                 dd[k].append(v)
 
-        return {k: v for k, v in dd.items() if len(v) > 1}
+        return {
+            # On a non-layered real dupe-calc (not calcing an input)
+            # overwrites would be +1 due to last `named_inputs` dict.
+            k: v
+            for k, v in dd.items()
+            if len(v) > 1
+        }
 
     def check_if_incomplete(self) -> Optional[IncompleteExecutionError]:
         """Return a :class:`IncompleteExecutionError` if `pipeline` operations failed/canceled. """
@@ -344,9 +422,7 @@ class Solution(ChainMap, Plottable):
         if incomplete:
             incomplete = list(yield_node_names(incomplete))
             partial_msgs = {
-                f"\n  +--{op.name}: {list(pouts)}"
-                for op, pouts in self.executed.items()
-                if pouts and isinstance(pouts, abc.Collection)
+                f"\n  +--{op.name}: {list(pouts)}" for op, pouts in self.broken.items()
             }
             err_msgs = [
                 f"\n  +--{op.name}: {type(ex).__name__}('{ex}')"
@@ -379,6 +455,15 @@ class Solution(ChainMap, Plottable):
         return plot_args
 
 
+#: Argument passed in a :term:`callbacks` callable before executing each operation,
+#: and contains fields to identify the operation call:
+#:
+#: :param op: the operation about to be computed;
+#: :param sol: the slution (might be just a plain dict if it has been marshalled);
+#: :param solid: the operation identity, needed if `sol` is a plain dict,
+OpCb = namedtuple("OpCb", "op, sol, solid")
+
+
 class _OpTask:
     """
     Mimic :class:`concurrent.futures.Future` for :term:`sequential` execution.
@@ -386,14 +471,25 @@ class _OpTask:
     This intermediate class is needed to solve pickling issue with process executor.
     """
 
-    __slots__ = ("op", "sol", "solid", "pre_callback", "result")
+    __slots__ = ("op", "sol", "solid", "callbacks", "result")
     logname = __name__
 
-    def __init__(self, op, sol, solid, pre_callback=None, result=UNSET):
+    def __init__(self, op, sol, solid, callbacks=None, result=UNSET):
         self.op = op
         self.sol = sol
         self.solid = solid
-        self.pre_callback = pre_callback
+
+        ## Make callbacks a 2-tuple with possible None callables.
+        #
+        if callbacks is None:
+            callbacks = (None, None)
+        elif callable(callbacks):
+            callbacks = (callbacks, None)
+        else:
+            callbacks = tuple(callbacks)
+            if len(callbacks) < 2:
+                callbacks = (*callbacks, None)
+        self.callbacks = callbacks
         self.result = result
 
     def marshalled(self):
@@ -407,10 +503,13 @@ class _OpTask:
             log = logging.getLogger(self.logname)
             log.debug("+++ (%s) Executing %s...", self.solid, self)
             token = task_context.set(self)
+            callbacks = self.callbacks
             try:
-                if self.pre_callback:
-                    self.pre_callback(OpCb(self.op, self.sol, self.solid))
+                if callbacks[0]:
+                    callbacks[0](OpCb(self.op, self.sol, self.solid))
                 self.result = self.op.compute(self.sol)
+                if callbacks[1]:
+                    callbacks[1](OpCb(self.op, self.sol, self.solid))
             finally:
                 task_context.reset(token)
 
@@ -455,15 +554,6 @@ def _do_task(task):
     return result
 
 
-#: Argument passed in a :term:`pre_callback` callable before executing each operation,
-#: and contains fields to identify the operation call:
-#:
-#: :param op: the operation about to be computed;
-#: :param sol: the slution (might be just a plain dict if it has been marshalled);
-#: :param solid: the operation identity, needed if `sol` is a plain dict,
-OpCb = namedtuple("OpCb", "op, sol, solid")
-
-
 class ExecutionPlan(
     namedtuple("ExecPlan", "net needs provides dag steps asked_outs comments"),  # noqa
     Plottable,
@@ -621,7 +711,7 @@ class ExecutionPlan(
                 # Mark start time here, to include also marshalling overhead.
                 solution.elapsed_ms[op] = time.time()
 
-                task = _OpTask(op, input_values, solution.solid, solution.pre_callback)
+                task = _OpTask(op, input_values, solution.solid, solution.callbacks)
                 if first_solid(global_marshal, getattr(op, "marshalled", None)):
                     task = task.marshalled()
 
@@ -691,7 +781,7 @@ class ExecutionPlan(
                 type(ex).__name__,
                 ex,
                 len(solution.executed),
-                solution.executed,
+                list(solution.executed),
                 exc_info=is_debug(),
             )
 
@@ -805,7 +895,7 @@ class ExecutionPlan(
                 if step in solution.canceled:
                     continue
 
-                task = _OpTask(step, solution, solution.solid, solution.pre_callback)
+                task = _OpTask(step, solution, solution.solid, solution.callbacks)
                 self._handle_task(task, step, solution)
 
             elif isinstance(step, str):
@@ -828,7 +918,7 @@ class ExecutionPlan(
         outputs=None,
         *,
         name="",
-        pre_callback: Callable[[OpCb], None] = None,
+        callbacks: Callable[[OpCb], None] = None,
         solution_class=None,
         layered_solution=None,
     ) -> Solution:
@@ -843,9 +933,10 @@ class ExecutionPlan(
             and scream if not.
         :param name:
             name of the pipeline used for logging
-        :param pre_callback:
-            If given, the :term:`pre_callback` is called before each operation,
-            with that as argument and solution.
+        :param callbacks:
+            If given, a 2-tuple with (optional) x2 :term:`callbacks` to call before & after
+            each operation, with :class:`.OpCb` as argument containing the op & solution.
+            Less or no elements accepted.
         :param solution_class:
             a custom solution factory to use
         :param layered_solution:
@@ -901,8 +992,8 @@ class ExecutionPlan(
                 {k: v for k, v in named_inputs.items() if k in dag.nodes}
                 if evict
                 else named_inputs,
-                pre_callback,
-                layered_solution=layered_solution,
+                callbacks,
+                is_layered=layered_solution,
             )
 
             log.info(
