@@ -10,18 +10,11 @@
 """
 
 import logging
-import textwrap
 import sys
+import textwrap
 from collections import abc as cabc
 from functools import update_wrapper, wraps
-from typing import (
-    Any,
-    Callable,
-    Collection,
-    List,
-    Mapping,
-    Tuple,
-)
+from typing import Any, Callable, Collection, List, Mapping, Tuple
 
 from boltons.setutils import IndexedSet as iset
 
@@ -43,6 +36,7 @@ from .modifier import (
     dep_renamed,
     dep_singularized,
     dep_stripped,
+    get_jsonp,
     get_keyword,
     is_implicit,
     is_optional,
@@ -51,6 +45,7 @@ from .modifier import (
     is_sfxed,
     is_vararg,
     is_varargs,
+    jsonp_ize,
     modify,
     optional,
 )
@@ -119,10 +114,6 @@ def as_renames(i, argname):
     return i
 
 
-def jsonp_ize(dep):
-    return modify(dep) if "/" in dep and type(dep) is str else dep
-
-
 def jsonp_ize_all(deps):
     """Auto-convert deps with slashes as :term:`jsonp` (unless ``no_jsonp``). """
     if deps:
@@ -170,25 +161,44 @@ def reparse_operation_data(
                 f"The `aliases` [{bad}] clash with existing provides in {list(provides)}!"
             )
 
+        # XXX: Why jsonp_ize here? (and not everywhere, or nowhere in fnop?)
         aliases = [(jsonp_ize(src), jsonp_ize(dst)) for src, dst in aliases]
-        alias_src = iset(src for src, _dst in aliases)
-        stripped_provides = iset(dep_stripped(d) for d in provides)
-        if not alias_src <= stripped_provides:
-            bad_alias_sources = alias_src - provides
+        aliases_src = iset(src for src, _dst in aliases)
+        all_provides = iset(provides) | (dep_stripped(d) for d in provides)
+
+        if not aliases_src <= all_provides:
+            bad_alias_sources = aliases_src - all_provides
             bad_aliases = ", ".join(
                 f"{src!r}-->{dst!r}" for src, dst in aliases if src in bad_alias_sources
             )
             raise ValueError(
-                f"The `aliases` [{bad_aliases}] rename non-existent provides in {list(stripped_provides)}!"
+                f"The `aliases` [{bad_aliases}] rename non-existent provides in {list(all_provides)}!"
             )
         sfx_aliases = [
-            f"{src} -> {dst}" for src, dst in aliases if is_sfx(src) or is_sfx(dst)
+            f"{src!r} -> {dst!r}"
+            for src, dst in aliases
+            if is_pure_sfx(src) or is_pure_sfx(dst)
         ]
         if sfx_aliases:
             raise ValueError(
                 f"The `aliases` must not contain `sideffects` {sfx_aliases}"
                 "\n  Simply add any extra `sideffects` in the `provides`."
             )
+        implicit_aliases = [
+            f"{'<implicit>' if bad_src else ''}{src!r} -> "
+            f"{dst!r}{'<implicit>' if bad_dst else ''}"
+            for src, dst in aliases
+            for bad_src in [
+                is_implicit(src) or any(is_implicit(i) for i in provides if i == src)
+            ]
+            for bad_dst in [is_implicit(dst)]
+            if bad_src or bad_dst
+        ]
+        if implicit_aliases:
+            raise ValueError(
+                f"The `aliases` must not contain `implicits`: {implicit_aliases}"
+                "\n  Simply add any extra `implicits` in the `provides`."
+            )
 
     return name, jsonp_ize_all(needs), jsonp_ize_all(provides), aliases
 
@@ -221,16 +231,16 @@ def _spread_sideffects(
     #: The dedupe  any `sideffected`.
     seen_sideffecteds = set()
 
-    def strip_unique(dep):
+    def as_fn_deps(dep):
         """Strip and dedupe any sfxed, drop any sfx and implicit. """
-        if is_implicit(dep):
+        if is_implicit(dep):  # must ignore also `sfxed`s
             pass
         elif is_sfxed(dep):
             dep = dep_stripped(dep)
             if not dep in seen_sideffecteds:
                 seen_sideffecteds.add(dep)
                 return (dep,)
-        elif not is_sfx(dep):
+        elif not is_sfx(dep):  # must kick after `sfxed`
             return (dep,)
         return ()
 
@@ -238,7 +248,7 @@ def _spread_sideffects(
 
     if deps:
         op_deps = iset(nn for n in deps for nn in dep_singularized(n))
-        fn_deps = tuple(nn for n in deps for nn in strip_unique(n))
+        fn_deps = tuple(nn for n in deps for nn in as_fn_deps(n))
         return op_deps, fn_deps
     else:
         return deps, deps
@@ -628,6 +638,12 @@ class FnOp(Operation):
 
                 keyword = get_keyword(n)
                 if keyword:
+                    ## Keep jsut the last part from `jsonp`s.
+                    #
+                    steps = get_jsonp(keyword)
+                    if steps:
+                        keyword = steps[-1]
+
                     kwargs[keyword] = inp_value
 
                 elif is_vararg(n):
@@ -805,7 +821,10 @@ class FnOp(Operation):
 
         if self.aliases:
             alias_values = [
-                (dst, results[src]) for src, dst in self.aliases if src in results
+                (dep_stripped(dst), results[stripped_src])
+                for src, dst in self.aliases
+                for stripped_src in [dep_stripped(src)]
+                if stripped_src in results
             ]
             results.update(alias_values)
 
