@@ -24,15 +24,17 @@ from typing import (
     Collection,
     Iterable,
     List,
-    Mapping,
+    MutableMapping,
     Optional,
-    Sequence,
+    MutableSequence,
     Tuple,
+    TypeVar,
     Union,
 )
 
 import numpy as np
 import pandas as pd
+from pandas.core.generic import NDFrame
 
 
 log = logging.getLogger(__name__)
@@ -130,7 +132,10 @@ class ResolveError(KeyError):
         return self.args[2]
 
 
-def _log_overwrite(part, doc, child):
+Doc = TypeVar("Doc", MutableMapping, MutableSequence)
+
+
+def _log_overwrite(part, doc: Doc, child):
     log.warning(
         "Overwritting json-pointer part %r in a %i-len subdoc over scalar %r.",
         part,
@@ -140,10 +145,10 @@ def _log_overwrite(part, doc, child):
 
 
 def resolve_path(
-    doc: Union[Sequence, Mapping],
+    doc: Doc,
     path: Union[str, Iterable[str]],
     default=UNSET,
-    root=UNSET,
+    root: Doc = UNSET,
     descend_objects=True,
 ):
     """
@@ -220,8 +225,8 @@ def resolve_path(
     :author: Julian Berman, ankostis
     """
     part_indexers = [
-        lambda doc, part: doc[int(part)],
         operator.getitem,
+        lambda doc, part: doc[int(part)],
         *((getattr,) if descend_objects else ()),
     ]
 
@@ -266,9 +271,9 @@ def resolve_path(
 
 
 def contains_path(
-    doc: Union[Sequence, Mapping],
+    doc: Doc,
     path: Union[str, Iterable[str]],
-    root=UNSET,
+    root: Doc = UNSET,
     descend_objects=True,
 ) -> bool:
     """Test if `doc` has a value for json-pointer path by calling :func:`.resolve_path()`. """
@@ -285,86 +290,134 @@ def is_collection(item):
     )
 
 
-def list_scouter(doc, part, container_factory, overwrite):
+def list_scouter(doc: Doc, idx, mother, overwrite) -> Tuple[Any, Optional[Doc]]:
     """
-    Get `doc `list item  by (int) `part`, or create a new one from `container_factory`.
+    Get `doc `list item  by (int) `idx`, or create a new one from `mother`.
 
-    NOTE: must kick before collection-scouter to handle special non-integer ``.`` index.
+    :param mother:
+        factory producing the child containers to extend missing steps,
+        or the "child" value (when `overwrite` is true).
+
+    :return:
+        a 2-tuple (child, ``None``)
+
+    NOTE: must come after collection-scouter due to special ``-`` index collision.
     """
-    if part == "-":  # It means the position after the last item.
-        item = container_factory()
+    if idx == "-":  # It means the position after the last item.
+        item = mother()
         doc.append(item)
-        return item
+        return item, None
 
-    part = int(part)  # Let it bubble, to try next (as key-item).
+    idx = int(idx)  # Let it bubble, to try next (as key-item).
     if overwrite and log.isEnabledFor(logging.WARNING):
         try:
-            child = doc[part]
+            child = doc[idx]
         except LookupError:
             # Out of bounds, but ok for now, will break also on assignment, below.
             pass
         else:
-            _log_overwrite(part, doc, child)
+            if not is_collection(child):
+                _log_overwrite(idx, doc, child)
     elif not overwrite:
         try:
-            child = doc[part]
+            child = doc[idx]
             if is_collection(child):
-                return child
+                return child, None
         except LookupError:
             # Ignore resolve errors, assignment errors below are more important.
             pass
 
-    item = container_factory()
-    doc[part] = item
-    return item
+    item = mother()
+    doc[idx] = item
+    return item, None
 
 
-def collection_scouter(doc, part, container_factory, overwrite):
-    """Get item `part` from `doc` collection, or create a new ome from `container_factory`."""
-    if overwrite and log.isEnabledFor(logging.WARNING) and part in doc:
-        _log_overwrite(part, doc, doc[part])
+def collection_scouter(
+    doc: Doc, key, mother, overwrite, concat_axis
+) -> Tuple[Any, Optional[Doc]]:
+    """
+    Get item `key` from `doc` collection, or create a new ome from `mother`.
+
+    :param mother:
+        factory producing the child containers to extend missing steps,
+        or the "child" value (when `overwrite` is true).
+
+    :return:
+        a 2-tuple (child, doc) where `doc` is not None if it needs to be replaced
+        in its parent container (e.g. due to df-concat with value).
+    """
+    if (
+        overwrite
+        and log.isEnabledFor(logging.WARNING)
+        and key in doc
+        and not is_collection(doc[key])
+    ):
+        _log_overwrite(key, doc, doc[key])
     elif not overwrite:
         try:
-            child = doc[part]
+            child = doc[key]
             if is_collection(child):
-                return child
+                return child, None
         except LookupError:
             # Ignore resolve errors, assignment errors below are more important.
             pass
 
-    item = container_factory()
-    doc[part] = item
-    return item
+    new_doc = None
+    value = mother()
+
+    if (
+        concat_axis is not None
+        and isinstance(doc, NDFrame)
+        and isinstance(value, NDFrame)
+    ):
+        new_doc = pd.concat((doc, value), axis=concat_axis)
+    else:
+        doc[key] = value
+
+    return value, new_doc
 
 
-def object_scouter(doc, part, value, container_factory, overwrite):
-    """Get attribute `part` in `doc` object, or create a new one from `container_factory`."""
-    if overwrite and log.isEnabledFor(logging.WARNING) and hasattr(doc, part):
-        _log_overwrite(part, doc, getattr(doc, part))
+def object_scouter(doc: Doc, attr, mother, overwrite) -> Tuple[Any, Optional[Doc]]:
+    """
+    Get attribute `attr` in `doc` object, or create a new one from `mother`.
+
+    :param mother:
+        factory producing the child containers to extend missing steps,
+        or the "child" value (when `overwrite` is true).
+
+    :return:
+        a 2-tuple (child, ``None``)
+    """
+    if overwrite and log.isEnabledFor(logging.WARNING) and hasattr(doc, attr):
+        _log_overwrite(attr, doc, getattr(doc, attr))
     elif not overwrite:
         try:
-            child = getattr(doc, part)
+            child = getattr(doc, attr)
             if is_collection(child):
-                return child
+                return child, None
         except AttributeError:
             # Ignore resolve errors, assignment errors below are more important.
             pass
 
-    item = container_factory()
-    setattr(doc, part, item)
-    return item
+    item = mother()
+    setattr(doc, attr, item)
+    return item, None
 
 
 def set_path_value(
-    doc: Union[Sequence, Mapping],
+    doc: Doc,
     path: Union[str, Iterable[str]],
     value,
     container_factory=dict,
-    root=UNSET,
+    root: Doc = UNSET,
     descend_objects=True,
+    concat_axis: int = None,
 ):
     """
-    Resolve a ``jsonpointer`` within the referenced ``doc``.
+    Set `value` into a :term:`jsonp` `path` within the referenced `doc`.
+
+    Special treatment (i.e. concat) if must insert a DataFrame into a DataFrame
+    with steps ``.``(vertical) and ``-``(horizontal) denoting concatenation axis.
 
     :param doc:
         the document to extend & insert `value`
@@ -372,7 +425,7 @@ def set_path_value(
         An absolute or relative json-pointer expression to resolve within `doc` document
         (or just the unescaped steps).
 
-        For sequences (arrays), it supports the special index dahs(``-``) char,
+        For sequences (arrays), it supports the special index dash(``-``) char,
         to refer to the position beyond the last item, as by the spec, BUT
         it does not raise - it always add a new item.
 
@@ -381,8 +434,8 @@ def set_path_value(
             https://tools.ietf.org/id/draft-handrews-relative-json-pointer-00.html
 
     :param container_factory:
-        a factory producing the collection instances to extend missing steps,
-        (usually a mapping or a sequence)
+        a factory producing the container to extend missing steps
+        (usually a mapping or a sequence).
     :param root:
         From where to start resolving absolute paths, double-slashes(``//``) or
         final slashes.
@@ -391,21 +444,29 @@ def set_path_value(
     :param descend_objects:
         If true, a last ditch effort is made for each part, whether it matches
         the name of an attribute of the parent item.
+    :param concat_axis:
+        if 0 or 1, applies :term:'pandas concatenation` vertically or horizontally,
+        by clipping last step when traversing it and doc & value are both Pandas objects.
 
-    :raises: ValueError (if jsonpointer empty, missing, invalid-content)
+    :raises ValueError:
+        - if jsonpointer empty, missing, invalid-content
+        - changed given `doc`/`root` (e.g due to concat-ed with value)
 
     See :func:`resolve_path()` for departures from the json-pointer standard
     """
 
     part_scouters = [
+        partial(collection_scouter, concat_axis=concat_axis),
         list_scouter,
-        collection_scouter,
         *((object_scouter,) if descend_objects else ()),
     ]
 
     if root is UNSET:
         root = doc
 
+    given_doc: Doc = doc  # Keep user input, to scream if must change (e.g. concat-ed).
+    parent: Doc = None
+    parent_part = None
     parts = jsonp_path(path) if isinstance(path, str) else list(path)
     last_part = len(parts) - 1
     for i, part in enumerate(parts):
@@ -415,33 +476,60 @@ def set_path_value(
                     f"Absolute step #{i} of json-pointer {path!r} without `root`!"
                 )
             doc = root
+            parent = parent_part = None
             continue
 
         if i == last_part:
             fact = lambda: value
             overwrite = True
+            if concat_axis is not None:
+                ## Only the last part supports concatenation.
+                part_scouters = part_scouters[:1]
         else:
             fact = container_factory
             overwrite = False
 
         for ii, scouter in enumerate(part_scouters):
             try:
-                doc = scouter(doc, part, fact, overwrite)
+                child, changed_doc = scouter(doc, part, fact, overwrite)
+
+                ## Replace doc/root upstream if changed
+                #  (e.g. df concat-ed with value).
+                #
+                if changed_doc is not None:
+                    if doc is root or doc is given_doc:
+                        # Changed-doc would be lost in vain...
+                        raise ValueError(
+                            f'Cannot modify given doc/root @ step (#{i}) "{part}" of path {path!r}:'
+                            f"\n  +--doc: {doc}"
+                            f"\n  +--root: {root}"
+                        )
+                    assert parent is not None and parent_part is not None, locals()
+
+                    parent[  # pylint: disable=unsupported-assignment-operation
+                        parent_part
+                    ] = changed_doc
+                    parent = changed_doc
+                else:
+                    parent = doc
+                parent_part = part
+                doc = child
                 break
             except Exception as ex:
-                if ii > 0:  # ignore int-indexing
-                    log.debug(
-                        "scouter %s failed on step (#%i)%s of json-pointer(%r) with doc(%s), due to: %s",
-                        scouter,
-                        i,
-                        part,
-                        path,
-                        doc,
-                        ex,
-                        # Don't log int(part) stack-traces.
-                        exc_info=type(ex) is not ValueError
-                        or not str(ex).startswith("invalid literal for int()"),
-                    )
+                if isinstance(ex, ValueError) and str(ex).startswith("Cannot modify"):
+                    raise
+                log.debug(
+                    "scouter %s failed on step (#%i)%s of json-pointer(%r) with doc(%s), due to: %s",
+                    scouter,
+                    i,
+                    part,
+                    path,
+                    doc,
+                    ex,
+                    # Don't log int(part) stack-traces.
+                    exc_info=type(ex) is not ValueError
+                    or not str(ex).startswith("invalid literal for int()"),
+                )
         else:
             raise ValueError(
                 f'Failed setting step (#{i}) "{part}" of path {path!r}!'
@@ -449,65 +537,148 @@ def set_path_value(
             )
 
 
-def _update_paths(
-    doc,
-    paths_vals: Collection[Tuple[List[str], Any]],
-    container_factory=dict,
-    root=UNSET,
-    descend_objects=True,
+def _index_or_delay_concat(
+    doc: Doc, key: str, value, delayed_concats: Optional[list]
 ) -> None:
-    # The `group` is a list of paths with common prefix (root)
-    # currently being built.
-    group_prefix, group = None, ()
-    for p, v in paths_vals + [((UNSET,), UNSET)]:
-        assert len(p) >= 1 or v is UNSET, locals()
+    """
+    Set Indexed value, or delay :term:`pandas concatenation`, for the recurse parent to do it.
 
-        next_prefix = p[0]
-        if next_prefix != group_prefix:
-            if len(p) == 1 and v is not UNSET:
-                # Assign value and proceed to the next one,
+    :param delayed_concats:
+        if given (not ``None``), pandas-concats are enabled and
+        should add further values into it
+        (may contain past values to be mass-concatenated by the caller)
+    """
+    if delayed_concats is None or not isinstance(doc, NDFrame):
+        doc[key] = value
+    else:
+        ## Delay further concats or Index non-pandas value.
+        #
+        if isinstance(value, NDFrame):
+            delayed_concats.append(value)
+        else:
+            assert not delayed_concats, f"Parent left delayed_concats? {locals()}"
+            doc[key] = value
+
+
+def _update_paths(
+    doc: Doc,
+    paths_vals: Collection[Tuple[List[str], Any]],
+    container_factory,
+    root: Doc,
+    descend_objects,
+    concat_axis,
+) -> Optional[Doc]:
+    """
+    (recursive) mass-update `path_vals` (jsonp, value) pairs into doc, with ..
+
+    special treatment for :term:`pandas concatenation`.
+
+    :return:
+        `doc` which might have changed, if it as a pandas concatenated.
+
+    FIXME: ROOT in mass-update_paths NOT IMPLEMENTED
+    FIXME: SET_OBJECT_ATTR in mass-update_paths NOT IMPLEMENTED
+    """
+    #: A `group` is a subset of the paths iterated below
+    #: that have a common "prefix", i.e. their 1st step,
+    #: seen as "root" for each recursed call.
+    group: List[Tuple[str, Any]] = ()  # Begin with a blocker value.
+    #: The `last_prefix` & `next_prefix` detect when group's 1st step
+    #: has changed while iterating (path, value) pairs
+    #: (meaning we have proceeded to the next `group)`.
+    last_prefix = None
+    #: Consecutive Pandas values to mass-concat.
+    delayed_concats: list = None if concat_axis is None else []
+    for i, (path, value) in enumerate((*paths_vals, ((UNSET,), UNSET))):
+        assert len(path) >= 1 or value is UNSET, locals()
+
+        ## Concate any delayed values.
+        #
+        if delayed_concats and (len(path) > 1 or not isinstance(value, NDFrame)):
+            assert concat_axis is not None and isinstance(
+                doc, NDFrame
+            ), f"Delayed without permission? {locals}"
+            doc = pd.concat((doc, *delayed_concats), axis=concat_axis)
+            delayed_concats = None
+
+        next_prefix = path[0]
+        if next_prefix != last_prefix:
+            if len(path) == 1 and value is not UNSET:
+                # Assign "tip" value of the before proceeding to the next group,
                 # THOUGH if a deeper path with this same prefix follows,
                 # it will overwrite the value just written.
-                doc[next_prefix] = v
+                _index_or_delay_concat(doc, next_prefix, value, delayed_concats)
             else:
-                if group_prefix:  # Is it past the 1st loop?
-                    ## Recurse into sub-group.
-                    #
+                if last_prefix:  # Is it past the 1st loop?
                     child = None
-                    if group_prefix in doc:
-                        child = doc[group_prefix]
+                    if last_prefix in doc:
+                        child = doc[last_prefix]
                         if not is_collection(child):
+                            _log_overwrite(last_prefix, doc, child)
                             child = None
-                            _log_overwrite(group_prefix, doc, child)
 
                     if child is None:
-                        child = doc[group_prefix] = container_factory()
-                    _update_paths(child, [(p[1:], v) for p, v in group])
+                        child = doc[last_prefix] = container_factory()
 
-                group_prefix, group = (next_prefix, [(p, v)])  # prepare the next group
+                    ## Recurse into collected sub-group.
+                    #
+                    sub_group = [(path[1:], value) for path, value in group]
+                    new_child = _update_paths(
+                        child,
+                        sub_group,
+                        container_factory,
+                        root,
+                        descend_objects,
+                        concat_axis,
+                    )
+                    if new_child is not None:
+                        doc[last_prefix] = new_child
+
+                # prepare the next group
+                last_prefix, group = (next_prefix, [(path, value)])
         else:
-            assert len(p) > 1, locals()  # shortest path switches group.
-            group.append((p, v))
+            assert len(path) > 1, locals()  # shortest path switches group.
+            group.append((path, value))  # pylint: disable=no-member
+
+    return doc
 
 
 def update_paths(
-    doc,
+    doc: Doc,
     paths_vals: Collection[Tuple[str, Any]],
     container_factory=dict,
-    root=UNSET,
+    root: Doc = UNSET,
     descend_objects=True,
+    concat_axis: int = None,
 ) -> None:
-    paths_vals = sorted(paths_vals)
-    _update_paths(
-        doc,
-        [(jsonp_path(p), v) for p, v in paths_vals],
-        container_factory,
-        root,
-        descend_objects,
+    """
+        Mass-update `path_vals` (jsonp, value) pairs into doc.
+
+        Group jsonp-keys by nesting level,to optimize.
+
+        :param concat_axis:
+            None, 0 or 1, see :func:`.set_path_value()`.
+    :return:
+            the updated doc (if it was a dataframe and ``pd.concact`` needed)
+    """
+    if root is UNSET:
+        root = doc
+    pvs = [(jsonp_path(p), v) for p, v in sorted(paths_vals, key=lambda pv: pv[0])]
+    new_doc = _update_paths(
+        doc, pvs, container_factory, root, descend_objects, concat_axis
     )
+    if new_doc is not doc:
+        # Changed-doc would be lost in vain...
+        raise ValueError(
+            f"Cannot mass-update Pandas @ ROOT:"
+            f"\n  +--(path, values): {pvs}"
+            f"\n  +--doc: {doc}"
+            f"\n  +--new_doc: {new_doc}"
+            + ("" if root is doc else f"\n  +--root: {root}")
+        )
 
 
-def list_popper(doc: Sequence, part, do_pop):
+def list_popper(doc: MutableSequence, part, do_pop):
     """Call :func:`collection_popper()` with integer `part`."""
     return collection_popper(doc, int(part), do_pop)
 
@@ -526,10 +697,10 @@ def object_popper(doc: Collection, part, do_pop):
 
 
 def pop_path(
-    doc: Union[Sequence, Mapping],
+    doc: Doc,
     path: Union[str, Iterable[str]],
     default=UNSET,
-    root=UNSET,
+    root: Doc = UNSET,
     descend_objects=True,
 ):
     """
