@@ -29,7 +29,7 @@ utilize a combination of these **diacritics**:
 .. diacritics-end
 """
 import enum
-from functools import lru_cache
+from functools import lru_cache, partial
 import operator
 from typing import (
     Any,
@@ -47,6 +47,7 @@ from typing import (
 #: Arguments-presence patterns for :class:`_Modifier` constructor.
 #: Combinations missing raise errors.
 _modifier_cstor_matrix = {
+# TODO: Add `implicit` in the table, to augment REPR & forbid implicit sfxed.
 # (7, kw, opt, accessors, sfxed, sfx): (STR, REPR, FUNC) OR None
 700000: None,
 710000: (       "%(dep)s",                  "'%(dep)s'(%(acs)s>%(kw)s)",    "keyword"),
@@ -133,8 +134,8 @@ class Accessor(NamedTuple):
 
 
 @lru_cache()
-def JsonpAccessor():
-    """Get/Set paths found on modifier's "extra' attribute `jsonpath` """
+def JsonpAcc():
+    """Read/write `jsonp` paths found on modifier's "extra' attribute `jsonpath` """
     from . import jsonpointer as jsp
 
     return Accessor(
@@ -146,6 +147,34 @@ def JsonpAccessor():
     )
 
 
+@lru_cache()
+def VCatAcc():
+    """Read/write `jsonp` and concat columns (axis=1) if both doc & value are Pandas. """
+    from . import jsonpointer as jsp
+
+    return Accessor(
+        contains=jsp.contains_path,
+        getitem=jsp.resolve_path,
+        setitem=partial(jsp.set_path_value, concat_axis=0),
+        delitem=jsp.pop_path,
+        update=partial(jsp.update_paths, concat_axis=0),
+    )
+
+
+@lru_cache()
+def HCatAcc():
+    """Read/write `jsonp` and concat columns (axis=1) if both doc & value are Pandas. """
+    from . import jsonpointer as jsp
+
+    return Accessor(
+        contains=jsp.contains_path,
+        getitem=jsp.resolve_path,
+        setitem=partial(jsp.set_path_value, concat_axis=1),
+        delitem=jsp.pop_path,
+        update=partial(jsp.update_paths, concat_axis=1),
+    )
+
+
 class _Modifier(str):
     """
     Annotate a :term:`dependency` with a combination of :term:`modifier`.
@@ -220,7 +249,14 @@ class _Modifier(str):
             accessor.validate()
         if sideffected and is_sfx(sideffected):
             raise ValueError(
-                f"`sideffected` cannot be sideffect, got {sideffected!r}"
+                f"`sideffected` cannot be `sfx`, got {sideffected!r}"
+                f"\n  locals={locals()}"
+            )
+
+        # TODO: Add `implicit` in the table, to augment REPR & forbid implicit sfxed.
+        if sideffected and kw.get("_implicit"):
+            raise ValueError(
+                f"`sideffected` cannot be `implicit`, got {sideffected!r}"
                 f"\n  locals={locals()}"
             )
         double_sideffects = [
@@ -298,17 +334,16 @@ def _modifier(
     It decides the final `name` and `_repr` for the new modifier by matching
     the given inputs with the :data:`_modifier_cstor_matrix`.
 
-    :param jsonp:
-        If given, it may be the pre-splitted *parts* of the :term:`jsonp`
-        dependency -- in that case, the dependency name is irrelevant -- or
-        a falsy (but not ``None``) value, to disable the automatic interpeting of
-        the dependency name as a json pointer path, regardless of any containing slashes.
     :param kw:
         extra kv pairs assigned as :class:`_Modifier` **private** attributes
         (prefixed with ``_`` if not already) only if values are not null.
         Here used for :term:`implicit`, and client code may extend its own modifiers.
     """
     if jsonp is not None:
+        if isinstance(jsonp, str):
+            from .jsonpointer import jsonp_path
+
+            jsonp = jsonp_path(jsonp)
         kw["_jsonp"] = jsonp  # WARN: must be False or a collection for jsonp-accessors!
     # Prevent sfx-jsonp.
     elif "/" in name and jsonp is None and (sideffected is None or sfx_list):
@@ -318,7 +353,7 @@ def _modifier(
     # Don't override user's accessor.
     #
     if not accessor and kw.get("_jsonp"):
-        accessor = JsonpAccessor()
+        accessor = JsonpAcc()
 
     args = (name, keyword, optional, accessor, sideffected, sfx_list)
     formats = _match_modifier_args(*args)
@@ -394,22 +429,219 @@ def modifier_withset(
     return _modifier(name=name, **kw)
 
 
-def modify(name: str, jsonp=None, implicit=None) -> _Modifier:
+def get_keyword(dep) -> Optional[str]:
+    """
+    Check if a :term:`dependency` is keyword (and get it, last step if `jsonp`).
+
+    All non-varargish optionals are "keyword" (including sideffected ones).
+
+    :return:
+        the :attr:`._keyword`
+    """
+    return getattr(dep, "_keyword", None)
+
+
+def is_optional(dep) -> Optional[_Optionals]:
+    """
+    Check if a :term:`dependency` is optional.
+
+    Varargish & optional sideffects are included.
+
+    :return:
+        the :attr:`._optional`
+    """
+    return getattr(dep, "_optional", None)
+
+
+def is_vararg(dep) -> bool:
+    """Check if an :term:`optionals` dependency is `vararg`."""
+    return getattr(dep, "_optional", None) == _Optionals.vararg
+
+
+def is_varargs(dep) -> bool:
+    """Check if an :term:`optionals` dependency is `varargs`."""
+    return getattr(dep, "_optional", None) == _Optionals.varargs
+
+
+def is_varargish(dep) -> bool:
+    """Check if an :term:`optionals` dependency is :term:`varargish`."""
+    return dep._optional in (_Optionals.vararg, _Optionals.vararg)
+
+
+def jsonp_ize(dep):
+    """Parse dep as :term:`jsonp` (unless it has been modified with ``jsnop=False``). """
+    return modify(dep) if "/" in dep and type(dep) is str else dep
+
+
+def get_jsonp(dep) -> Union[List[str], None]:
+    """Check if dependency is :term:`json pointer path` and return its steps."""
+    return getattr(jsonp_ize(dep), "_jsonp", None)
+
+
+def is_sfx(dep) -> Optional[str]:
+    """
+    Check if a dependency is :term:`sideffects` or :term:`sideffected`.
+
+    :return:
+        the :attr:`._sideffected`
+    """
+    return getattr(dep, "_sideffected", None)
+
+
+def is_pure_sfx(dep) -> bool:
+    """Check if it is :term:`sideffects` but not a :term:`sideffected`."""
+    return getattr(dep, "_sideffected", None) and not getattr(dep, "_sfx_list", None)
+
+
+def is_sfxed(dep) -> bool:
+    """
+    Check if it is :term:`sideffected`.
+
+    :return:
+        the :attr:`._sfx_list` if it is  a *sideffected* dep, None/empty-tuple otherwise
+    """
+    return getattr(dep, "_sideffected", None) and getattr(dep, "_sfx_list", None)
+
+
+def is_implicit(dep) -> bool:
+    """Return if it is a :term:`implicit` dependency. """
+    return getattr(dep, "_implicit", None)
+
+
+def get_accessor(dep) -> bool:
+    """
+    Check if dependency has an :term:`accessor`, and get it (if funcs below are unfit)
+
+    :return:
+        the :attr:`._accessor`
+    """
+    return getattr(dep, "_accessor", None)
+
+
+def acc_contains(dep) -> Callable[[Collection, str], Any]:
+    """
+    A fn like :func:`operator.contains` for any `dep` (with-or-without :term:`accessor`)
+    """
+    acc = getattr(dep, "_accessor", None)
+    return acc.contains if acc else operator.contains
+
+
+def acc_getitem(dep) -> Callable[[Collection, str], Any]:
+    """
+    A fn like :func:`operator.getitem` for any `dep` (with-or-without :term:`accessor`)
+    """
+    acc = getattr(dep, "_accessor", None)
+    return acc.getitem if acc else operator.getitem
+
+
+def acc_setitem(dep) -> Callable[[Collection, str, Any], None]:
+    """
+    A fn like :func:`operator.setitem` for any `dep` (with-or-without :term:`accessor`)
+    """
+    acc = getattr(dep, "_accessor", None)
+    return acc.setitem if acc else operator.setitem
+
+
+def acc_delitem(dep) -> Callable[[Collection, str], None]:
+    """
+    A fn like :func:`operator.delitem` for any `dep` (with-or-without :term:`accessor`)
+    """
+    acc = getattr(dep, "_accessor", None)
+    return acc.delitem if acc else operator.delitem
+
+
+def dependency(dep) -> str:
+    """
+    Returns the underlying dependency name (just str)
+
+    For non-sideffects, it coincides with str(), otherwise,
+    the the pure-sideffect string or the existing :term:`sideffected` dependency
+    stored in :attr:`._sideffected`.
+    """
+    return str(dep) if is_sfx(dep) else dep._sideffected
+
+
+def dep_renamed(dep, ren) -> Union[_Modifier, str]:
+    """
+    Renames `dep` as `ren` or call `ren`` (if callable) to decide its name,
+
+    preserving any :func:`keyword` to old-name.
+
+    For :term:`sideffected` it renames the dependency (not the *sfx-list*) --
+    you have to do it that manually with a custom renamer-function, if ever
+    the need arise.
+    """
+    if callable(ren):
+        renamer = ren
+    else:
+        renamer = lambda n: ren
+
+    if isinstance(dep, _Modifier):
+        if is_sfx(dep):
+            new_name = renamer(dep._sideffected)
+            dep = modifier_withset(dep, name=new_name, sideffected=new_name)
+        else:
+            dep = modifier_withset(dep, name=renamer(str(dep)))
+    else:  # plain string
+        dep = renamer(dep)
+
+    return dep
+
+
+def dep_singularized(dep) -> Iterable[Union[str, _Modifier]]:
+    """
+    Return one sideffected for each sfx in :attr:`._sfx_list`, or iterate `dep` in other cases.
+    """
+    sfx_list = is_sfxed(dep)
+    return (
+        [modifier_withset(dep, sfx_list=(s,)) for s in sfx_list] if sfx_list else (dep,)
+    )
+
+
+def dep_stripped(dep) -> Union[str, _Modifier]:
+    """
+    Return the :attr:`._sideffected` if `dep` is :term:`sideffected`, `dep` otherwise,
+
+    conveying all other properties of the original modifier to the stripped dependency.
+    """
+    if is_sfxed(dep):
+        dep = modifier_withset(
+            dep, name=dep._sideffected, sideffected=None, sfx_list=()
+        )
+    return dep
+
+
+############
+## Announced API
+## (exported with "*" in package.__init___)
+##
+
+
+def modify(
+    name: str, *, keyword=None, jsonp=None, implicit=None, accessor: Accessor = None
+) -> _Modifier:
     """
     Generic :term:`modifier` for term:`json pointer path` & :term:`implicit` dependencies.
 
     :param jsonp:
-        If given, it may be the pre-splitted *parts* of the json pointer path
-        for the dependency -- in that case, the dependency name is irrelevant -- or
-        a falsy (but not ``None``) value, to disable the automatic interpeting of
-        the dependency name as a json pointer path, regardless of any containing slashes.
+        If given, it may be some other json-pointer expression, or
+        the pre-splitted *parts* of the :term:`jsonp` dependency -- in that case,
+        the dependency name is irrelevant -- or a falsy (but not ``None``) value,
+        to disable the automatic interpeting of the dependency name as a json pointer path,
+        regardless of any containing slashes.
 
-        .. Tip::
-            If accessing pandas, you may pass an already splitted path with
-            its last *part* being a `callable indexer
-            <https://pandas.pydata.org/pandas-docs/stable/user_guide/indexing.html#selection-by-callable>`_.
+        If accessing pandas, you may pass an already splitted path with
+        its last *part* being a callable indexer (:ref:`pandas:indexing.callable`).
+
+        In addition to writing values, the :func:`.vcat` or :func:`.hcat` modifiers
+        (& respective accessors) support also `pandas concatenation` for `provides`
+        (see example in :ref:`jsnop-df-concat`).
     :param implicit:
         :term:`implicit` dependencies are not fed into/out of the function.
+        You may use directly :func:`implicit`.
+    :param accessor:
+        Annotate the `dependency` with :term:`accessor` functions to read/write
+        `solution` (actually a 2-tuple with functions is ok)
 
     **Example:**
 
@@ -464,7 +696,24 @@ def modify(name: str, jsonp=None, implicit=None) -> _Modifier:
 
     .. graphtik::
     """
-    return _modifier(name, jsonp=jsonp, implicit=implicit)
+    return _modifier(
+        name, keyword=keyword, jsonp=jsonp, implicit=implicit, accessor=accessor
+    )
+
+
+def implicit(name, *, jsonp=None) -> _Modifier:
+    """see :term:`implicit` & generic :func:`.modify` modifier. """
+    return _modifier(name, implicit=True, jsonp=jsonp)
+
+
+def vcat(name, *, keyword: str = None, jsonp=None) -> _Modifier:
+    """Provides-only, see :term:`pandas concatenation` & generic :func:`.modify` modifier. """
+    return _modifier(name, accessor=VCatAcc(), keyword=keyword, jsonp=jsonp)
+
+
+def hcat(name, *, keyword: str = None, jsonp=None) -> _Modifier:
+    """Provides-only, see :term:`pandas concatenation` & generic :func:`.modify` modifier. """
+    return _modifier(name, accessor=HCatAcc(), keyword=keyword, jsonp=jsonp)
 
 
 def keyword(
@@ -500,15 +749,8 @@ def keyword(
         the functions to access values to/from solution (see :class:`Accessor`)
         (actually a 2-tuple with functions is ok)
     :param jsonp:
-        If given, it may be the pre-splitted *parts* of the json pointer path
-        for the dependency -- in that case, the dependency name is irrelevant -- or
-        a falsy (but not ``None``) value, to disable the automatic interpeting of
-        the dependency name as a json pointer path, regardless of any containing slashes.
-
-        .. Tip::
-            If accessing pandas, you may pass an already splitted path with
-            its last *part* being a `callable indexer
-            <https://pandas.pydata.org/pandas-docs/stable/user_guide/indexing.html#selection-by-callable>`_.
+        None (derrived from `name`), ``False``, str, collection of str/callable (last one)
+        See generic :func:`.modify` modifier.
 
     :return:
         a :class:`_Modifier` instance, even if no `keyword` is given OR
@@ -566,7 +808,7 @@ def keyword(
 
 
 def optional(
-    name: str, keyword: str = None, accessor: Accessor = None, jsonp=None
+    name: str, keyword: str = None, accessor: Accessor = None, jsonp=None, implicit=None
 ) -> _Modifier:
     """
     Annotate :term:`optionals` `needs` corresponding to *defaulted* op-function arguments, ...
@@ -585,15 +827,11 @@ def optional(
         the functions to access values to/from solution (see :class:`Accessor`)
         (actually a 2-tuple with functions is ok)
     :param jsonp:
-        If given, it may be the pre-splitted *parts* of the json pointer path
-        for the dependency -- in that case, the dependency name is irrelevant -- or
-        a falsy (but not ``None``) value, to disable the automatic interpeting of
-        the dependency name as a json pointer path, regardless of any containing slashes.
-
-        .. Tip::
-            If accessing pandas, you may pass an already splitted path with
-            its last *part* being a `callable indexer
-            <https://pandas.pydata.org/pandas-docs/stable/user_guide/indexing.html#selection-by-callable>`_.
+        None (derrived from `name`), ``False``, str, collection of str/callable (last one)
+        See generic :func:`.modify` modifier.
+    :param implicit:
+        :term:`implicit` dependencies are not fed into/out of the function.
+        You may use directly :func:`implicit`.
 
     **Example:**
 
@@ -641,36 +879,10 @@ def optional(
         optional=_Optionals.keyword,
         accessor=accessor,
         jsonp=jsonp,
+        implicit=implicit,
     )
 
 
-def accessor(name: str, accessor: Accessor = None, jsonp=None) -> _Modifier:
-    """
-    Annotate a `dependency` with :term:`accessor` functions to read/write `solution`.
-
-    :param accessor:
-        the functions to access values to/from solution (see :class:`Accessor`)
-        (actually a 2-tuple with functions is ok)
-    :param jsonp:
-        If given, it may be the pre-splitted *parts* of the json pointer path
-        for the dependency -- in that case, the dependency name is irrelevant -- or
-        a falsy (but not ``None``) value, to disable the automatic interpeting of
-        the dependency name as a json pointer path, regardless of any containing slashes.
-
-        .. Tip::
-            If accessing pandas, you may pass an already splitted path with
-            its last *part* being a `callable indexer
-            <https://pandas.pydata.org/pandas-docs/stable/user_guide/indexing.html#selection-by-callable>`_.
-
-
-    - Probably not very usefull -- see the :func:`.modify` modifier for an integrated
-      use case.
-    - To combine it with `optional`, `keyword`, etc use the rest modifier factories
-      and pass an argument value to their `accessor` parameter.
-    """
-    return _modifier(name, accessor=accessor, jsonp=jsonp)
-
-
 def vararg(name: str, accessor: Accessor = None, jsonp=None) -> _Modifier:
     """
     Annotate a :term:`varargish` `needs` to  be fed as function's ``*args``.
@@ -679,16 +891,8 @@ def vararg(name: str, accessor: Accessor = None, jsonp=None) -> _Modifier:
         the functions to access values to/from solution (see :class:`Accessor`)
         (actually a 2-tuple with functions is ok)
     :param jsonp:
-        If given, it may be the pre-splitted *parts* of the json pointer path
-        for the dependency -- in that case, the dependency name is irrelevant -- or
-        a falsy (but not ``None``) value, to disable the automatic interpeting of
-        the dependency name as a json pointer path, regardless of any containing slashes.
-
-        .. Tip::
-            If accessing pandas, you may pass an already splitted path with
-            its last *part* being a `callable indexer
-            <https://pandas.pydata.org/pandas-docs/stable/user_guide/indexing.html#selection-by-callable>`_.
-
+        None (derrived from `name`), ``False``, str, collection of str/callable (last one)
+        See generic :func:`.modify` modifier.
 
     .. seealso::
         Consult also the example test-case in: :file:`test/test_op.py:test_varargs()`,
@@ -738,16 +942,8 @@ def varargs(name: str, accessor: Accessor = None, jsonp=None) -> _Modifier:
         the functions to access values to/from solution (see :class:`Accessor`)
         (actually a 2-tuple with functions is ok)
     :param jsonp:
-        If given, it may be the pre-splitted *parts* of the json pointer path
-        for the dependency -- in that case, the dependency name is irrelevant -- or
-        a falsy (but not ``None``) value, to disable the automatic interpeting of
-        the dependency name as a json pointer path, regardless of any containing slashes.
-
-        .. Tip::
-            If accessing pandas, you may pass an already splitted path with
-            its last *part* being a `callable indexer
-            <https://pandas.pydata.org/pandas-docs/stable/user_guide/indexing.html#selection-by-callable>`_.
-
+        None (derrived from `name`), ``False``, str, collection of str/callable (last one)
+        See generic :func:`.modify` modifier.
 
     .. seealso::
         Consult also the example test-case in: :file:`test/test_op.py:test_varargs()`,
@@ -892,14 +1088,13 @@ def sfxed(
     optional: bool = None,
     accessor: Accessor = None,
     jsonp=None,
-    implicit=None,
 ) -> _Modifier:
     r"""
     Annotates a :term:`sideffected` dependency in the solution sustaining side-effects.
 
     :param dependency:
         the actual dependency receiving the sideffect, which will be fed into/out
-        of the function (unless marked as :term:`implicit`).
+        of the function.
     :param sfx0:
         the 1st (arbitrary object) sideffect marked as "acting" on the `dependency`.
     :param sfx0:
@@ -913,17 +1108,8 @@ def sfxed(
         the functions to access values to/from solution (see :class:`Accessor`)
         (actually a 2-tuple with functions is ok)
     :param jsonp:
-        If given, it may be the pre-splitted *parts* of the json pointer path
-        for the dependency -- in that case, the dependency name is irrelevant -- or
-        a falsy (but not ``None``) value, to disable the automatic interpeting of
-        the dependency name as a json pointer path, regardless of any containing slashes.
-
-        .. Tip::
-            If accessing pandas, you may pass an already splitted path with
-            its last *part* being a `callable indexer
-            <https://pandas.pydata.org/pandas-docs/stable/user_guide/indexing.html#selection-by-callable>`_.
-    :param implicit:
-        :term:`implicit` dependencies are not fed into/out of the function.
+        None (derrived from `name`), ``False``, str, collection of str/callable (last one)
+        See generic :func:`.modify` modifier.
 
     Like :func:`.sfx` but annotating a *real* :term:`dependency` in the solution,
     allowing that dependency to be present both in :term:`needs` and :term:`provides`
@@ -1014,7 +1200,6 @@ def sfxed(
         sfx_list=(sfx0, *sfx_list),
         accessor=accessor,
         jsonp=jsonp,
-        implicit=implicit,
     )
 
 
@@ -1044,180 +1229,3 @@ def sfxed_varargs(
         accessor=accessor,
         jsonp=jsonp,
     )
-
-
-def get_keyword(dep) -> Optional[str]:
-    """
-    Check if a :term:`dependency` is keyword (and get it).
-
-    All non-varargish optionals are "keyword" (including sideffected ones).
-
-    :return:
-        the :attr:`._keyword`
-    """
-    return getattr(dep, "_keyword", None)
-
-
-def is_optional(dep) -> Optional[_Optionals]:
-    """
-    Check if a :term:`dependency` is optional.
-
-    Varargish & optional sideffects are included.
-
-    :return:
-        the :attr:`._optional`
-    """
-    return getattr(dep, "_optional", None)
-
-
-def is_vararg(dep) -> bool:
-    """Check if an :term:`optionals` dependency is `vararg`."""
-    return getattr(dep, "_optional", None) == _Optionals.vararg
-
-
-def is_varargs(dep) -> bool:
-    """Check if an :term:`optionals` dependency is `varargs`."""
-    return getattr(dep, "_optional", None) == _Optionals.varargs
-
-
-def is_varargish(dep) -> bool:
-    """Check if an :term:`optionals` dependency is :term:`varargish`."""
-    return dep._optional in (_Optionals.vararg, _Optionals.vararg)
-
-
-def get_jsonp(dep) -> Union[List[str], None]:
-    """Check if the dependency is :term:`json pointer path` and return steps."""
-    return getattr(dep, "_jsonp", None)
-
-
-def is_sfx(dep) -> Optional[str]:
-    """
-    Check if a dependency is :term:`sideffects` or :term:`sideffected`.
-
-    :return:
-        the :attr:`._sideffected`
-    """
-    return getattr(dep, "_sideffected", None)
-
-
-def is_pure_sfx(dep) -> bool:
-    """Check if it is :term:`sideffects` but not a :term:`sideffected`."""
-    return getattr(dep, "_sideffected", None) and not getattr(dep, "_sfx_list", None)
-
-
-def is_sfxed(dep) -> bool:
-    """
-    Check if it is :term:`sideffected`.
-
-    :return:
-        the :attr:`._sfx_list` if it is  a *sideffected* dep, None/empty-tuple otherwise
-    """
-    return getattr(dep, "_sideffected", None) and getattr(dep, "_sfx_list", None)
-
-
-def is_implicit(dep) -> bool:
-    """Return if it is a :term:`implicit` dependency. """
-    return getattr(dep, "_implicit", None)
-
-
-def get_accessor(dep) -> bool:
-    """
-    Check if dependency has an :term:`accessor`, and get it (if funcs below are unfit)
-
-    :return:
-        the :attr:`._accessor`
-    """
-    return getattr(dep, "_accessor", None)
-
-
-def acc_contains(dep) -> Callable[[Collection, str], Any]:
-    """
-    A fn like :func:`operator.contains` for any `dep` (with-or-without :term:`accessor`)
-    """
-    acc = getattr(dep, "_accessor", None)
-    return acc.contains if acc else operator.contains
-
-
-def acc_getitem(dep) -> Callable[[Collection, str], Any]:
-    """
-    A fn like :func:`operator.getitem` for any `dep` (with-or-without :term:`accessor`)
-    """
-    acc = getattr(dep, "_accessor", None)
-    return acc.getitem if acc else operator.getitem
-
-
-def acc_setitem(dep) -> Callable[[Collection, str, Any], None]:
-    """
-    A fn like :func:`operator.setitem` for any `dep` (with-or-without :term:`accessor`)
-    """
-    acc = getattr(dep, "_accessor", None)
-    return acc.setitem if acc else operator.setitem
-
-
-def acc_delitem(dep) -> Callable[[Collection, str], None]:
-    """
-    A fn like :func:`operator.delitem` for any `dep` (with-or-without :term:`accessor`)
-    """
-    acc = getattr(dep, "_accessor", None)
-    return acc.delitem if acc else operator.delitem
-
-
-def dependency(dep) -> str:
-    """
-    Returns the underlying dependency name (just str)
-
-    For non-sideffects, it coincides with str(), otherwise,
-    the the pure-sideffect string or the existing :term:`sideffected` dependency
-    stored in :attr:`._sideffected`.
-    """
-    return str(dep) if is_sfx(dep) else dep._sideffected
-
-
-def dep_renamed(dep, ren) -> Union[_Modifier, str]:
-    """
-    Renames `dep` as `ren` or call `ren`` (if callable) to decide its name,
-
-    preserving any :func:`keyword` to old-name.
-
-    For :term:`sideffected` it renames the dependency (not the *sfx-list*) --
-    you have to do it that manually with a custom renamer-function, if ever
-    the need arise.
-    """
-    if callable(ren):
-        renamer = ren
-    else:
-        renamer = lambda n: ren
-
-    if isinstance(dep, _Modifier):
-        if is_sfx(dep):
-            new_name = renamer(dep._sideffected)
-            dep = modifier_withset(dep, name=new_name, sideffected=new_name)
-        else:
-            dep = modifier_withset(dep, name=renamer(str(dep)))
-    else:  # plain string
-        dep = renamer(dep)
-
-    return dep
-
-
-def dep_singularized(dep) -> Iterable[Union[str, _Modifier]]:
-    """
-    Return one sideffected for each sfx in :attr:`._sfx_list`, or iterate `dep` in other cases.
-    """
-    sfx_list = is_sfxed(dep)
-    return (
-        [modifier_withset(dep, sfx_list=(s,)) for s in sfx_list] if sfx_list else (dep,)
-    )
-
-
-def dep_stripped(dep) -> Union[str, _Modifier]:
-    """
-    Return the :attr:`._sideffected` if `dep` is :term:`sideffected`, `dep` otherwise,
-
-    conveying all other properties of the original modifier to the stripped dependency.
-    """
-    if is_sfxed(dep):
-        dep = modifier_withset(
-            dep, name=dep._sideffected, sideffected=None, sfx_list=()
-        )
-    return dep
