@@ -367,7 +367,7 @@ class Pipeline(Operation):
         recompute_from: Items = None,
         *,
         predicate: "NodePredicate" = UNSET,
-        pre_callback=None,
+        callbacks=None,
         solution_class: "Type[Solution]" = None,
         layered_solution=None,
     ) -> "Solution":
@@ -401,19 +401,20 @@ class Pipeline(Operation):
         :param predicate:
             filter-out nodes before compiling
             If not given, those set by a previous call to :meth:`withset()` or cstor are used.
-        :param pre_callback:
-            If given, the :term:`pre_callback` is called before each operation,
-            with that as argument and solution.
+        :param callbacks:
+            If given, a 2-tuple with (optional) x2 :term:`callbacks` to call before & after
+            each operation, with :class:`.OpCb` as argument containing the op & solution.
+            Less or no elements accepted.
         :param solution_class:
             a custom solution factory to use
         :param layered_solution:
-            whether to store operation results into separate :term:`solution layer`\\s
+            whether to store operation results or just keys into separate :term:`solution layer`\\s
 
             Unless overridden by a True/False in :func:`.set_layered_solution`
             of :term:`configurations`, it accepts the following values:
 
-            - When True(False), always keep(don't keep) results in a separate layer for each operation,
-              regardless of any *jsonp* dependencies.
+            - When True(False), always keep results(just the keys) in a separate
+              layer for each operation, regardless of any *jsonp* dependencies.
             - If ``None``, layers are used only if there are NO :term:`jsonp` dependencies
               in the network.
 
@@ -469,7 +470,7 @@ class Pipeline(Operation):
                 named_inputs,
                 outputs,
                 name=self.name,
-                pre_callback=pre_callback,
+                callbacks=callbacks,
                 solution_class=solution_class,
                 layered_solution=layered_solution,
             )
@@ -496,9 +497,9 @@ class Pipeline(Operation):
                         "Suppressed error while logging/plotting jetsam of %s: %s(%s)"
                         "\n  +--annotations:%s",
                         self,
+                        type(ex2).__name__,
+                        ex2,
                         jetsam,
-                        type(ex).__name__,
-                        ex,
                         exc_info=True,
                     )
 
