@@ -391,10 +391,20 @@ def _render_template(tpl: jinja2.Template, **kw) -> str:
 def make_data_value_tooltip(plot_args: PlotArgs):
     """Called on datanodes, when solution exists. """
     node = plot_args.nx_item
-    if node in plot_args.solution:
-        val = plot_args.solution.get(node)
-        tooltip = "(None)" if val is None else f"({type(val).__name__}) {val}"
-        return quote_html_tooltips(tooltip)
+    assert node in plot_args.solution
+    val = plot_args.solution[node]
+    tooltip = "(None)" if val is None else f"({type(val).__name__}) {val}"
+    return quote_html_tooltips(tooltip)
+
+
+def make_overwrite_tooltip(plot_args: PlotArgs):
+    """Called on datanodes, withmultiple overwrite values. """
+    node = plot_args.nx_item
+    assert node in plot_args.solution
+    val = plot_args.solution.overwrites[node]
+    val_str = "\n  ".join(f"{i}. {v}" for i, v in reversed(list(enumerate(val))))
+    tooltip = f"(x{len(val)} overwrites) {val_str}"
+    return quote_html_tooltips(tooltip)
 
 
 def make_op_prune_comment(plot_args: PlotArgs):
@@ -403,7 +413,7 @@ def make_op_prune_comment(plot_args: PlotArgs):
     plottable = plot_args.plottable
 
     comments = ()
-    if sol is not None and op in sol.plan.comments:
+    if hasattr(sol, "plan") and op in sol.plan.comments:
         comments = sol.plan.comments
     elif op in getattr(plottable, "comments", ()):
         comments = plottable.comments
@@ -553,7 +563,7 @@ class Theme:
     kw_data_overwritten = {
         "style": ["filled"],
         "fillcolor": Ref("overwrite_color"),
-        "tooltip": ["(overwritten)"],
+        "tooltip": [make_overwrite_tooltip],
     }
     kw_data_missing = {
         "fontcolor": Ref("canceled_color"),
@@ -587,7 +597,7 @@ class Theme:
                 {%- endif -%}
 
                 {%- if nx_item | sfxed -%}
-                    <BR/>(<I>sfx:</I> {{ nx_item | sfx_list | join(', ') | eee }})
+                    <BR/>(<I>sfxed:</I> {{ nx_item | sfx_list | join(', ') | eee }})
                 {%- endif -%}
 
             </TD>
@@ -1437,7 +1447,7 @@ class Plotter:
             #
             is_pruned = (
                 hasattr(plottable, "dag") and nx_node not in plottable.dag.nodes
-            ) or (solution is not None and nx_node not in solution.dag.nodes)
+            ) or (hasattr(solution, "dag") and nx_node not in solution.dag.nodes)
             if is_pruned:
                 graph.nodes[nx_node]["_pruned"] = True  # Signal to edge-plotting.
                 styles.add("kw_data_pruned")
@@ -1453,7 +1463,7 @@ class Plotter:
                 if nx_node in solution:
                     styles.add("kw_data_in_solution")
 
-                    if nx_node in solution.overwrites:
+                    if nx_node in getattr(solution, "overwrites", ()):
                         styles.add("kw_data_overwritten")
 
                 elif nx_node in steps:
@@ -1500,11 +1510,11 @@ class Plotter:
             label_styles.add("kw_op_prune_comment")
 
             if solution:
-                if solution.is_failed(nx_node):
+                if hasattr(solution, "is_failed") and solution.is_failed(nx_node):
                     label_styles.add("kw_op_failed")
-                elif nx_node in solution.executed:
+                elif nx_node in getattr(solution, "executed", ()):
                     label_styles.add("kw_op_executed")
-                elif nx_node in solution.canceled:
+                elif nx_node in getattr(solution, "canceled", ()):
                     label_styles.add("kw_op_canceled")
 
             label_styles.stack_user_style(node_attrs)
@@ -1618,7 +1628,8 @@ class Plotter:
         if graph.nodes[src].get("_pruned") or graph.nodes[dst].get("_pruned"):
             styles.add("kw_edge_pruned")
         if (
-            solution is not None
+            hasattr(solution, "dag")
+            and hasattr(solution, "plan")
             and (src, dst) not in solution.dag.edges
             and (src, dst) in solution.plan.dag.edges
         ):
