@@ -13,6 +13,7 @@ import pytest
 from graphtik import (
     base,
     compose,
+    implicit,
     modify,
     operation,
     pipeline,
@@ -273,19 +274,19 @@ def test_jetsam_n_plot_with_DEBUG():
             needs=[
                 "a",
                 sfxed("b", "foo", keyword="bb"),
-                modify("c", implicit=1),
-                sfxed("d", "bar", implicit=1),
+                implicit("c"),
+                sfxed("d", "bar"),
                 vararg("e"),
                 varargs("f"),
             ],
             provides=[
                 "A",
                 sfxed("b", "FOO", keyword="bb"),
-                modify("C", implicit=1),
-                sfxed("d", "BAR", implicit=1),
+                implicit("C"),
+                sfxed("d", "BAR", optional=True),
                 sfx("FOOBAR"),
             ],
-            aliases={"A": "aaa", "b": "bbb", "C": "ccc", "d": "ddd"},
+            aliases={"A": "aaa", "b": "bbb", "d": "ddd"},  # FIXME: "D" is implicit!
         ),
     )
 
