@@ -12,6 +12,7 @@ import pytest
 from graphtik import (
     NO_RESULT,
     compose,
+    implicit,
     modify,
     operation,
     optional,
@@ -136,22 +137,20 @@ def test_cycle_tip():
         pipe.compute()
 
 
-def test_aliases(exemethod):
-    provides = ("a", sfxed("s1", "foo"), sfxed("s2", "foo", implicit=1))
+def test_aliases_pipeline(exemethod):
+    provides = ("a", sfxed("s", "foo"))
     aliased = operation(
         lambda: ("A", "B"),
         name="op1",
         provides=provides,
-        aliases={"a": "b", "s1": "S1", "s2": "S2"},
+        aliases={"a": "b", "s": "S1"},
     )
     assert aliased._user_provides == provides
     assert tuple(aliased.provides) == (
         "a",
-        sfxed("s1", "foo"),
-        sfxed("s2", "foo"),
+        sfxed("s", "foo"),
         "b",
         "S1",
-        "S2",
     )
 
     pipe = compose(
@@ -160,7 +159,7 @@ def test_aliases(exemethod):
         operation(lambda x: x * 2, name="op2", needs="b", provides="c"),
         parallel=exemethod,
     )
-    assert pipe() == {"a": "A", "s1": "B", "b": "A", "S1": "B", "c": "AA"}
+    assert pipe() == {"a": "A", "s": "B", "b": "A", "S1": "B", "c": "AA"}
     assert list(pipe.provides) == [*aliased.provides, "c"]
 
 
@@ -866,6 +865,8 @@ def test_execution_endurance(exemethod, endurance, endured):
         sol = graph(**inp)
         assert sol.is_failed(scream1) and sol.is_failed(scream2)
         assert "Must not have run!" in str(sol.executed[scream1])
+        assert sol.is_failed(scream1) == sol.executed[scream1]
+        assert sol.is_failed(scream2) == sol.executed[scream2]
         assert sol == {"a+b": 3, "a+2b": 5, **inp}
         assert "x2 failures" in str(sol.check_if_incomplete())
         with pytest.raises(IncompleteExecutionError, match="x2 failures"):
@@ -952,7 +953,7 @@ def test_pre_callback(quarantine_pipeline, exemethod):
         assert op_cb.sol["quarantine"]
         called_ops.append(op_cb.op.name)
 
-    sol = pipeline.compute({"quarantine": True}, pre_callback=op_called)
+    sol = pipeline.compute({"quarantine": True}, callbacks=op_called)
     assert sol == {
         "quarantine": True,
         "time": "1h",
@@ -1164,7 +1165,6 @@ def test_recompute_till():
 
     with evictions_skipped(True):
         sol = pipe.compute(inp, outputs="a3", recompute_from="a1")
-        sol.plot("t.pdf")
         assert exe_ops(sol) == ["f1", "f2"]
         assert sol == {"a0": 1, "a1": 3, "a2": 6, "a3": 12, "a4": 16}
 
