@@ -12,7 +12,7 @@ from graphtik import NO_RESULT, compose, operation, sfxed, vararg
 from graphtik.base import RenArgs
 from graphtik.config import solution_layered
 from graphtik.execution import task_context
-from graphtik.modifier import accessor, dep_renamed, modify
+from graphtik.modifier import dep_renamed, modify
 
 
 pytestmark = pytest.mark.usefixtures("log_levels")
@@ -25,8 +25,8 @@ def test_solution_accessor_simple():
         "copy values in solution: a+b-->A+BB",
         operation(
             (lambda *a: a),
-            needs=[accessor("a", acc), accessor("b", acc)],
-            provides=[accessor("A", acc), accessor("BB", acc)],
+            needs=[modify("a", accessor=acc), modify("b", accessor=acc)],
+            provides=[modify("A", accessor=acc), modify("BB", accessor=acc)],
         ),
     )
     sol = copy_values.compute({"a": 1, "b": 2})
@@ -38,7 +38,7 @@ def test_jsonp_disabled():
         fn=None,
         name="copy a+b-->A+BB",
         needs=[modify("inputs/a"), modify("inputs/b")],
-        provides=["RESULTS/A", modify("RESULTS/BB", False)],
+        provides=["RESULTS/A", modify("RESULTS/BB", jsonp=False)],
     )
     assert "provides=['RESULTS/A'($), RESULTS/BB" in str(no_jsonp_res)
     res = compose("", no_jsonp_res).compute({"inputs": {"a": 1, "b": 2}})
@@ -46,7 +46,7 @@ def test_jsonp_disabled():
 
     no_jsonp_inp = operation(
         name="copy a+b-->A+BB",
-        needs=["inputs/a", modify("inputs/b", False)],
+        needs=["inputs/a", modify("inputs/b", jsonp=False)],
         provides=["RESULTS/A", "RESULTS/BB"],
     )()
     assert "needs=['inputs/a'($), inputs/b]" in str(no_jsonp_inp)
@@ -410,7 +410,9 @@ def test_network_nest_subdocs_NOT_LAYERED(solution_layered_false):
         "todos": range(14, 17),
     }
 
-    assert sol.overwrites == {}
+    assert sol.overwrites == {
+        "backlog": [range(14, 17), range(9, 17), range(4, 17), range(0, 17)]
+    }
 
     ## -1 tasks for Wednesday to enact
 
@@ -427,8 +429,7 @@ def test_network_nest_subdocs_NOT_LAYERED(solution_layered_false):
         "weekly_tasks": (range(0, 4), range(4, 9)),
         "todos": (),
     }
-    assert sol.overwrites == {}
-
+    assert sol.overwrites == {"backlog": [range(9, 9), range(4, 9), range(0, 9)]}
     sol = week.compute(
         {"backlog": range(9)},
         outputs=["backlog", "daily_tasks", "weekly_tasks", "todos"],
