@@ -15,6 +15,7 @@ from graphtik import (
     NO_RESULT,
     NO_RESULT_BUT_SFX,
     compose,
+    implicit,
     keyword,
     operation,
     optional,
@@ -301,7 +302,10 @@ def test_pipeline_node_props():
     op2 = operation(lambda: None, name="b", node_props={"a": 3, "c": 4})
     pipeline = compose("n", op1, op2, node_props={"bb": 22, "c": 44})
 
-    exp = {"a": {"a": 11, "b": 0, "bb": 22, "c": 44}, "b": {"a": 3, "bb": 22, "c": 44}}
+    exp = {
+        "a": {"typ": 1, "a": 11, "b": 0, "bb": 22, "c": 44},
+        "b": {"typ": 1, "a": 3, "bb": 22, "c": 44},
+    }
     node_props = _collect_op_props(pipeline)
     assert node_props == exp
 
@@ -321,7 +325,10 @@ def test_pipeline_merge_node_props():
     pipeline = compose(
         "n", pipeline1, pipeline2, node_props={"bb": 22, "c": 44}, nest=False
     )
-    exp = {"a": {"a": 1, "bb": 22, "c": 44}, "b": {"a": 3, "bb": 22, "c": 44}}
+    exp = {
+        "a": {"typ": 1, "a": 1, "bb": 22, "c": 44},
+        "b": {"typ": 1, "a": 3, "bb": 22, "c": 44},
+    }
     node_props = _collect_op_props(pipeline)
     assert node_props == exp
 
@@ -329,9 +336,9 @@ def test_pipeline_merge_node_props():
         "n", pipeline1, pipeline2, node_props={"bb": 22, "c": 44}, nest=True
     )
     exp = {
-        "n1.a": {"a": 1, "bb": 22, "c": 44},
-        "n2.a": {"a": 11, "b": 0, "bb": 22, "c": 44},
-        "n2.b": {"a": 3, "bb": 22, "c": 44},
+        "n1.a": {"typ": 1, "a": 1, "bb": 22, "c": 44},
+        "n2.a": {"typ": 1, "a": 11, "b": 0, "bb": 22, "c": 44},
+        "n2.b": {"typ": 1, "a": 3, "bb": 22, "c": 44},
     }
     node_props = _collect_op_props(pipeline)
     assert node_props == exp
@@ -391,6 +398,16 @@ def test_as_renames(inp, exp):
             (["a", "b"], {"a": "b"}),
             ValueError(r"clash with existing provides in \['a', 'b'\]"),
         ),
+        ## Implicits
+        #
+        (
+            ([implicit("i")], {implicit("i"): "II"}),
+            ValueError("must not contain `implicits"),
+        ),
+        (
+            (["i"], {"i": implicit("II")}),
+            ValueError("must not contain `implicits"),
+        ),
     ],
 )
 def test_func_op_validation_aliases_BAD(prov_aliases, ex):
@@ -398,10 +415,37 @@ def test_func_op_validation_aliases_BAD(prov_aliases, ex):
         reparse_operation_data("t", None, *prov_aliases)
 
 
-def test_provides_aliases():
-    op = operation(str, name="t", needs="s", provides="a", aliases={"a": "aa"})
-    assert op.provides == {"a", "aa"}
-    assert op.compute({"s": "k"}) == {"a": "k", "aa": "k"}
+def test_keyword_jsonp():
+    op = operation(lambda *, bar: bar * 2, "kw", needs=keyword("foo/bar"), provides="a")
+    sol = op.compute({"foo/bar": "ciao"})
+    assert sol == {"a": "ciaociao"}
+
+
+@pytest.mark.parametrize(
+    "provide, aliases, exp",
+    [
+        ("a", {"a": "aa"}, {"a": "k", "aa": "k"}),
+        (
+            sfxed("a", "1"),
+            {sfxed("a", "1"): "aa"},
+            {"a": "k", "aa": "k"},
+        ),
+        (
+            "a",
+            {"a": sfxed("a", "1")},
+            {"a": "k"},
+        ),
+        (
+            sfxed("a", "1"),
+            {sfxed("a", "1"): sfxed("b", "1")},
+            {"a": "k", "b": "k"},
+        ),
+    ],
+)
+def test_provides_aliases(provide, aliases, exp):
+    op = operation(str, name="t", needs="s", provides=provide, aliases=aliases)
+    assert op.provides == aliases.keys() | aliases.values()
+    assert op.compute({"s": "k"}) == exp
 
 
 def test_sfxed_needs_in_pipeline():
